#include <stdio.h>
#include <string.h>

#define NB_OUTILS 2

int outil_help (int, char **);
int outil_cat (int, char **); // in cat.c

struct outil {
    char *name;
    int (*func)(int, char **);
} outils[NB_OUTILS] = {
    { "help", outil_help },
    { "cat", outil_cat },
};

int
outil_help (int argc, char **argv)
{
    if (strcmp(argv[0], "help") != 0) { // not explicitly called
        if (argc > 1) { // unknown command
            fprintf(stderr, "%s: error: '%s' command not found.\n", argv[0], argv[1]);
            return 1;
        }
        else { // no command given
            fprintf(stderr, "Usage: %s command [arguments]\n", argv[0]);
        }
    }
    fprintf(stderr, "Available commands:\n");
    for (int i = 0; i < NB_OUTILS; i++) {
        fprintf(stderr, "\t%s\n", outils[i].name);
    }
    return 1;
}

int
find_outil (const char *cmd)
{
    for (int i = 0; i < NB_OUTILS; i++) {
        if (strcmp(cmd, outils[i].name) == 0) return i;
    }
    return -1;
}

int
main (int argc, char *argv[])
{
    if (argc < 2) return outil_help(argc, argv);
    int outil = find_outil(argv[1]);
    if (outil < 0) return outil_help(argc, argv);
    return outils[outil].func(argc - 1, argv + 1);
}
